# Generated by roxygen2: do not edit by hand

S3method(print,profvis)
export(parse_rprof)
export(pause)
export(profvis)
export(profvisOutput)
export(profvis_server)
export(profvis_ui)
export(renderProfvis)
import(htmlwidgets)
import(rlang)
importFrom(utils,Rprof)
useDynLib(profvis, .registration = TRUE, .fixes = "c_")
