% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pause.R
\name{pause}
\alias{pause}
\title{Pause an R process}
\usage{
pause(seconds)
}
\arguments{
\item{seconds}{Number of seconds to pause.}
}
\description{
This function pauses an R process for some amount of time. It differs from
\code{\link[=Sys.sleep]{Sys.sleep()}} in that time spent in \code{pause} will show up in
profiler data. Another difference is that \code{pause} uses up 100\\% of a CPU,
whereas \code{Sys.sleep} does not.
}
\examples{
# Wait for 0.5 seconds
pause(0.5)

}
